@@ -3,7 +3,7 @@ package Net::FullAuto::ISets::Amazon::Chaining_is;
 ### OPEN SOURCE LICENSE - GNU AFFERO PUBLIC LICENSE Version 3.0 #######
 #
 #    Net::FullAuto - Powerful Network Process Automation Software
-#    Copyright © 2000-2015  Brian M. Kelly
+#    Copyright © 2000-2016  Brian M. Kelly
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU Affero General Public License as
@@ -183,6 +183,9 @@ my $configure_chaining=sub {
       '__display__');
    ($stdout,$stderr)=$handle->cwd('zeromq-4.1.3');
    cmd_raw($handle,'export PKG_CONFIG_PATH=/usr/local/lib/pkgconfig');
+   my $ad="Defaults    env_keep += \"PKG_CONFIG_PATH\"";
+   ($stdout,$stderr)=$handle->cmd(
+      "sudo sed -i \'/_XKB_CHARSET/a$ad\' /etc/sudoers");
    ($stdout,$stderr)=$handle->cmd('./configure','__display__');
    ($stdout,$stderr)=$handle->cmd('make','__display__');
    ($stdout,$stderr)=$handle->cmd('sudo make install','__display__');
@@ -321,6 +324,7 @@ END
       Proc::ProcessTable
       Parallel::Forker
       UUID::Tiny
+      Regexp::Assemble
 
    );
    my $install_chaining=<<'END';
@@ -365,7 +369,8 @@ END
       }
       cmd_raw($handle,
          "sudo LD_LIBRARY_PATH=/usr/local/lib ".
-         "LD_PRELOAD=/usr/local/lib/libzmq.so.5 cpan $module",
+         "PKG_CONFIG_PATH=/usr/local/lib/pkgconfig ".
+         "LD_PRELOAD=/usr/local/lib/libzmq.so.5 cpan -f -i $module",
          '__display__');
    }
    $show=<<END;
@@ -375,11 +380,11 @@ END
 
 ########################################
 END
-   print $show;
-   cmd_raw($handle,
-      'sudo perl -MCPAN -e \'CPAN::Shell->force('.
-      '"install","Regexp::Assemble")\'',
-      '__display__');
+#   print $show;
+#   cmd_raw($handle,
+#      'sudo perl -MCPAN -e \'CPAN::Shell->force('.
+#      '"install","Regexp::Assemble")\'',
+#      '__display__');
    $show=<<END;
 ########################################
 
@@ -418,6 +423,16 @@ END
    $show=<<END;
 ########################################
 
+   INSTALLING CatalystX::OAuth2
+
+########################################
+END
+   cmd_raw($handle,'sudo cpan CatalystX::OAuth2',
+      '__display__');
+   print "\n";
+   $show=<<END;
+########################################
+
    INSTALLING Task::Catalyst::Tutorial
 
 ########################################
@@ -616,6 +631,45 @@ END
    ($stdout,$stderr)=$handle->cwd("../../..");
    ($stdout,$stderr)=$handle->cmd("./script/adventrest_create.pl ".
       "model DB DBIC::Schema AdventREST::Schema",'__display__');
+   ($stdout,$stderr)=$handle->cmd("./script/adventrest_create.pl ".
+      "controller OAuth2::Provider",'__display__');
+   my $pro_path="./lib/AdventREST/Controller/OAuth2/Provider.pm";
+   ($stdout,$stderr)=$handle->cmd('sudo sed -i '.
+      "\"s/Catalyst::Controller/Catalyst::Controller::ActionRole/\" ".
+      $pro_path);
+   $ad="%NL%".
+       "with %SQ%CatalystX::OAuth2::Controller::Role::Provider%SQ%;%NL%".
+       "%NL%".
+       "__PACKAGE__->config(%NL%".
+       "  store => {%NL%".
+       "    class => %SQ%DBIC%SQ%,%NL%".
+       "    client_model => %SQ%DB::Client%SQ%%NL%".
+       "  }%NL%".
+       ");%NL%".
+       "%NL%".
+       "sub request : Chained(%SQ%/%SQ%) Args(0) ".
+       "Does(%SQ%OAuth2::RequestAuth%SQ%) {}%NL%".
+       "%NL%".
+       "sub grant : Chained(%SQ%/%SQ%) Args(0) ".
+       "Does(%SQ%OAuth2::GrantAuth%SQ%) {%NL%".
+       "  my ( \$self, \$c ) = \@_;%NL%".
+       "%NL%".
+       "  my \$oauth2 = \$c->req->oauth2;%NL%".
+       "%NL%".
+       "  \$c->user_exists and \$oauth2->user_is_valid(1)%NL%".
+       "    or \$c->detach(%SQ%/passthrulogin%SQ%);%NL%".
+       "}%NL%".
+       "%NL%".
+       "sub token : Chained(%SQ%/%SQ%) Args(0) ".
+       "Does(%SQ%OAuth2::AuthToken::ViaAuthGrant%SQ%) {}%NL%".
+       "%NL%".
+       "sub refresh : Chained(%SQ%/%SQ%) Args(0) ".
+       "Does(%SQ%OAuth2::AuthToken::ViaRefreshToken%SQ%) {}%NL%";
+   ($stdout,$stderr)=$handle->cmd(
+      "sudo sed -i '/ActionRole/a$ad' $pro_path");
+   ($stdout,$stderr)=$handle->cmd( # bash shell specific
+      "sudo sed -i \'s/%NL%/\'\"`echo \\\\\\n`/g\" $pro_path");
+   ($stdout,$stderr)=$handle->cmd("sudo sed -i \"s/%SQ%/\'/g\" $pro_path");
    $content=<<'END';
 name: AdventREST
 Model::DB:
@@ -668,6 +722,17 @@ use Moose;
 use TheSchwartz::Job;
 use DBI;
 use namespace::autoclean;
+use ZMQ::LibZMQ4;
+use ZMQ::Constants qw(:all);
+use YAML;
+use Carp::Assert;
+
+use constant NBR_WORKERS    => 2;
+use constant READY          => \\x22\\\\001\\x22;
+
+use constant FRONTEND_URL   =>
+       'ipc:///home/ec2-user/AdventREST/frontend.ipc';
+
 BEGIN { extends 'Catalyst::Controller::REST' }
 
 __PACKAGE__->config(
@@ -702,6 +767,27 @@ sub user_list_GET {
        \\x24page=int(\\x24start/\\x24per_page)+1;
     }
 
+    my \\x24id = 'Client-'.\\x24\\x24;
+
+    my \\x24ctx     = zmq_init();
+    my \\x24socket  = zmq_socket(\\x24ctx,ZMQ_REQ);
+
+    my \\x24rv      = zmq_setsockopt(\\x24socket,ZMQ_IDENTITY,\\x24id);
+    assert(\\x24rv == 0, 'setting socket options');
+
+    \\x24rv         = zmq_connect(\\x24socket,FRONTEND_URL());
+
+    assert(\\x24rv == 0,'connecting client ...');
+
+    print \\x22\\x24id sending Hello\\\\n\\x22;
+    \\x24rv         = zmq_msg_send('Hello',\\x24socket);
+
+    my \\x24reply = zmq_recvmsg(\\x24socket);
+
+    assert(\\x24reply);
+
+    print \\x22\\x24id got a reply -> \\x22,zmq_msg_data(\\x24reply),\\x22\\n\\x22;
+
     # We'll use an array now:
     my @user_list;
     my \\x24rs = \\x24c->model('DB::User')
@@ -822,7 +908,7 @@ END
    ($stdout,$stderr)=$handle->cmd("touch User.pm");
    ($stdout,$stderr)=$handle->cmd("chmod -v 777 User.pm",'__display__');
    ($stdout,$stderr)=$handle->cmd("echo -e \"$content\" > User.pm");
-   my $ad='sub index : Private {%NL%   my ( $self, $c ) = @_;%NL%'.
+   $ad='sub index : Private {%NL%   my ( $self, $c ) = @_;%NL%'.
           '   $c->forward( $c->view(%SQ%TT%SQ%) );%NL%}%NL%%NL%';
    ($stdout,$stderr)=$handle->cmd(
       "sudo sed -i '/sub index :Path/i$ad' ./Root.pm");
@@ -1000,14 +1086,24 @@ Model::TheSchwartz:
     databases:
       - dsn: dbi:SQLite:__path_to(db/theschwartz.sqlt)__
 END
-   ($stdout,$stderr)=$handle->cmd("echo \"$content\" >> adventrest.yml"); 
-   $ad="use Parallel::Forker;%NL%".
+   ($stdout,$stderr)=$handle->cmd("echo \"$content\" >> adventrest.yml");
+   $ad="use Net::FullAuto;%NL%".
+       "use ZMQ::LibZMQ4;%NL%".
+       "use ZMQ::Constants qw(:all);%NL%".
+       "use YAML;%NL%".
+       "use Carp::Assert;%NL%%NL%".
+       "use constant NBR_WORKERS    => 2;%NL%".
+       "use constant READY          => \"\\\\001\";%NL%".
+       "%NL%".
+       "use constant BACKEND_URL    =>%NL%".
+       "       %SQ%ipc:///home/ec2-user/AdventREST/backend.ipc%SQ%;%NL%".
+       "use constant FRONTEND_URL    =>%NL%".
+       "       %SQ%ipc:///home/ec2-user/AdventREST/frontend.ipc%SQ%;%NL%".
+       "%NL%".
+       "use Parallel::Forker;%NL%".
        "%NL%".
        "my \$fa_sub=sub {%NL%".
        "%NL%".
-       "   use Net::FullAuto;%NL%".
-       "   use ZMQ::LibZMQ4;%NL%".
-       "   use YAML;%NL%".
        "   my \$server={%NL%".
        "%NL%".
        "      Label => %SQ%server%SQ%,%NL%".
@@ -1016,33 +1112,177 @@ END
        "      HostName => %SQ%localhost%SQ%,%NL%".
        "%NL%".
        "   };%NL%".
-       "   # A bit of custom config riding/hacking to use the application%SQ%s%NL%".
+       "   # A bit of custom config riding/hacking to use the application".
+       "%SQ%s%NL%".
        "   # config for the DB.%NL%".
-       "   my \$config_file = %SQ%/home/ec2-user/AdventREST/adventrest.yml%SQ%;%NL%".
+       "   my \$config_file = %SQ%/home/ec2-user/AdventREST/adventrest.yml".
+       "%SQ%;%NL%".
        "   my \$config_data = YAML::LoadFile( \$config_file );%NL%".
        "   #my \$args = \$config_data->{\"Model::ZeroMQ\"}->{args};%NL%".
        "%NL%".
+       "   my \$id = %SQ%Worker-%SQ%.\$\$;%NL%%NL%".
+       "   my \$ctx     = zmq_init();%NL%".
+       "   my \$socket  = zmq_socket(\$ctx,ZMQ_REQ);%NL%%NL%".
+       "   my \$rv      = zmq_setsockopt(\$socket,ZMQ_IDENTITY,\$id);%NL%".
+       "   assert(\$rv == 0);%NL%%NL%".
+       "   \$rv         = zmq_connect(\$socket,BACKEND_URL());%NL%".
+       "   assert(\$rv == 0,%SQ%connecting to backend%SQ%);%NL%%NL%".
+       "   print \"\$id sending READY\\n\";%NL%".
+       "%NL%".
+       "   \$rv = zmq_msg_send(READY(),\$socket);%NL%".
+       "   assert(\$rv);%NL%".
+       "%NL%".
+       "   my \$buf = zmq_msg_init();%NL%".
+       "%NL%".
        "   my (\$fullauto,\$stdout,\$stderr,\$exitcode,".
        "\$connect_error)=(%SQ%%SQ%,%SQ%%SQ%,%SQ%%SQ%,%SQ%%SQ%,%SQ%%SQ%);%NL%".
        "   (\$fullauto,\$connect_error)=connect_ssh(\$server);%NL%".
        "%NL%".
-       "   use Time::HiRes qw( clock_nanosleep TIMER_ABSTIME );%NL%".
+       "   use Time::HiRes;%NL%".
        "   # http://www.unitconversion.org/unit_converter/time-ex.html%NL%".
+       "   my \$msg = zmq_msg_init();%NL%".
        "   while (1) {%NL%".
        "%NL%".
-       "      clock_nanosleep(CLOCK_REALTIME, 5e4, TIMER_ABSTIME);%NL%".
+       "      my \@msg=();%NL%".
+       "      if (zmq_msg_recv(\$buf,\$socket,ZMQ_DONTWAIT)) {%NL%".
+       "         push \@msg, zmq_msg_data(\$buf);%NL%".
+       "         while (zmq_getsockopt(\$socket,ZMQ_RCVMORE)) {%NL%".
+       "            zmq_msg_recv(\$buf,\$socket);%NL%".
+       "            push \@msg, zmq_msg_data(\$buf);%NL%".
+       "         }%NL%".
+       "      }%NL%".
        "      (\$stdout,\$stderr)=\$fullauto->cmd(%SQ%hostname%SQ%);%NL%".
-       "      print \"LOOPING in process \$\$ and ".
-       "JOB=\$job and HOSTNAME=\$stdout\\n\";%NL%".
+       "      #print \"LOOPING in process \$\$ and ".
+       "and HOSTNAME=\$stdout\\n\";%NL%".
        "      if (getppid==1) {%NL%".
        "         `pgrep -P \$\$ | xargs kill -TERM`;%NL%".
        "         exit;%NL%".
        "      }%NL%".
+       "      if (\$#msg) {%NL%".
+       "         print \"\$id got: \$msg[2] from \$msg[0]\\n\";%NL%".
+       "         print \"\$id sending OK to \$msg[0]\\n\";%NL%".
+       "         zmq_msg_send(\$msg[0],\$socket,ZMQ_SNDMORE);%NL%".
+       "         zmq_msg_send(%SQ%%SQ%,\$socket,ZMQ_SNDMORE);%NL%".
+       "         zmq_msg_send(%SQ%OK%SQ%,\$socket);%NL%".
+       "      }%NL%".
+       "      Time::HiRes::sleep (.000001);%NL%".
        "%NL%".
        "   }%NL%".
        "};%NL%".
        "%NL%".
-       "for (1..5) {%NL%".
+       "my \$zeromq_broker=sub {%NL%".
+       "%NL%".
+       "   print \"LOADING BROKER\\n\";%NL%".
+       "%NL%".
+       "   my \$ctx = zmq_init();%NL%".
+       "   my \$frontend = zmq_socket(\$ctx, ZMQ_ROUTER);%NL%".
+       "   my \$backend  = zmq_socket(\$ctx, ZMQ_ROUTER);%NL%".
+       "%NL%".
+       "   my \$rv  = zmq_bind(\$frontend,FRONTEND_URL());%NL%".
+       "   assert(\$rv == 0);%NL%".
+       "%NL%".
+       "   \$rv     = zmq_bind(\$backend,BACKEND_URL());%NL%".
+       "   assert(\$rv == 0);%NL%".
+       "%NL%".
+       "   my \@workers;%NL%".
+       "%NL%".
+       "   my (\$w_addr,\$delim,\$c_addr,\$data);%NL%".
+       "   my \$items = [%NL%".
+       "%NL%".
+       "         {%NL%".
+       "            events      => ZMQ_POLLIN,%NL%".
+       "            socket      => \$frontend,%NL%".
+       "            callback    => sub {%NL%".
+       "%NL%".
+       "                if (-1<\$#workers) {%NL%".
+       "%NL%".
+       "                    print \"frontend…\\n\";%NL%".
+       "%NL%".
+       "                    my \$buf = zmq_msg_init();%NL%".
+       "                    my \@msg=();%NL%".
+       "                    if (zmq_msg_recv(\$buf,\$frontend)) {%NL%".
+       "                       push \@msg, zmq_msg_data(\$buf);%NL%".
+       "                       while (zmq_getsockopt(\$frontend,ZMQ_RCVMORE)) {%NL%".
+       "                          zmq_msg_recv(\$buf,\$frontend);%NL%".
+       "                          push \@msg, zmq_msg_data(\$buf);%NL%".
+       "                       }%NL%".
+       "                    }%NL%".
+       "%NL%".
+       "                    assert(\$#msg);%NL%".
+       "%NL%".
+       "                    assert(\$#workers < NBR_WORKERS());%NL%".
+       "%NL%".
+       "                    zmq_msg_send(pop(\@workers),\$backend,ZMQ_SNDMORE);%NL%".
+       "                    zmq_msg_send(%SQ%%SQ%,\$backend,ZMQ_SNDMORE);%NL%".
+       "                    zmq_msg_send(\$msg[0],\$backend,ZMQ_SNDMORE);%NL%".
+       "                    zmq_msg_send(%SQ%%SQ%,\$backend,ZMQ_SNDMORE);%NL%".
+       "                    zmq_msg_send(\$msg[2],\$backend,);%NL%".
+       "%NL%".
+       "                }%NL%".
+       "            },%NL%".
+       "         },%NL%".
+       "         {%NL%".
+       "            events      => ZMQ_POLLIN,%NL%".
+       "            socket      => \$backend,%NL%".
+       "            callback    => sub {%NL%".
+       "%NL%".
+       "                print \"backend…\\n\";%NL%".
+       "%NL%".
+       "                my \$buf = zmq_msg_init();%NL%".
+       "                my \@msg=();%NL%".
+       "                if (zmq_msg_recv(\$buf,\$backend)) {%NL%".
+       "                   push \@msg, zmq_msg_data(\$buf);%NL%".
+       "                   while (zmq_getsockopt(\$backend,ZMQ_RCVMORE)) {%NL%".
+       "                      zmq_msg_recv(\$buf,\$backend);%NL%".
+       "                      push \@msg, zmq_msg_data(\$buf);%NL%".
+       "                   }%NL%".
+       "                }%NL%".
+       "%NL%".
+       "                assert(\$#msg);%NL%".
+       "%NL%".
+       "                \$w_addr = \$msg[0];%NL%".
+       "                push(\@workers,\$w_addr);%NL%".
+       "%NL%".
+       "                \$delim = \$msg[1];%NL%".
+       "                assert(\$delim eq %SQ%%SQ%);%NL%".
+       "%NL%".
+       "                \$c_addr = \$msg[2];%NL%".
+       "%NL%".
+       "                if(\$c_addr ne READY()){%NL%".
+       "                    \$delim = \$msg[3];%NL%".
+       "                    assert (\$delim eq %SQ%%SQ%);%NL%".
+       "%NL%".
+       "                    \$data = \$msg[4];%NL%".
+       "%NL%".
+       "                    print %SQ%sending %SQ%.\$data.%SQ% to %SQ%.\$c_addr.\"\\n\";%NL%".
+       "%NL%".
+       "                    zmq_msg_send(\$c_addr,\$frontend,ZMQ_SNDMORE);%NL%".
+       "                    zmq_msg_send(%SQ%%SQ%,\$frontend,ZMQ_SNDMORE);%NL%".
+       "                    zmq_msg_send(\$data,\$frontend);%NL%".
+       "%NL%".
+       "                } else {%NL%".
+       "                    print %SQ%worker checking in: %SQ%.\$w_addr.\"\\n\";%NL%".
+       "                }%NL%".
+       "            },%NL%".
+       "         },%NL%".
+       "   ];%NL%".
+       "   while(1){%NL%".
+       "      zmq_poll(\$items);%NL%".
+       "      select undef,undef,undef,0.025;%NL%".
+       "      if (getppid==1) {%NL%".
+       "         `pgrep -P \$\$ | xargs kill -TERM`;%NL%".
+       "          exit;%NL%".
+       "      }%NL%".
+       "   }%NL%".
+       "%NL%".
+       "};%NL%".
+       "%NL%".
+       "my \$Fork = new Parallel::Forker();%NL%".
+       "\$Fork->schedule(run_on_start => \$zeromq_broker)->run()%NL%".
+       "   if \$Fork->in_parent;%NL%".
+       "setpgrp(0,0) unless \$Fork->in_parent;%NL%".
+       "%NL%".
+       "for (1..NBR_WORKERS()) {%NL%".
        "%NL%".
        "   my \$Fork = new Parallel::Forker();%NL%".
        "   #\$SIG{TERM} = sub { \$Fork->kill_tree_all(%SQ%TERM%SQ%) ".
@@ -1086,6 +1326,7 @@ END
    $handle->{_cmd_handle}->print(
       'script/myapp_server.pl --background');
    ($stdout,$stderr)=$handle->cwd("../AdventREST");
+   ($stdout,$stderr)=$handle->cmd('sudo ldconfig');
    $handle->{_cmd_handle}->print(
       'sudo script/adventrest_server.pl -p 3001 &');
    $prompt=substr($handle->{_cmd_handle}->prompt(),1,-1);
@@ -1107,6 +1348,7 @@ END
    my ($hash,$output,$error)=('','','');
    ($hash,$output,$error)=run_aws_cmd($c);
    my $mdns=$hash->{Reservations}->[0]->{Instances}->[0]->{PublicDnsName};
+   sleep 15;
    print "\n   ACCESS CATALYST REST EXAMPLE AT:\n\n",
          " http://$mdns:3001\n";
    my $thanks=<<'END';
@@ -1124,7 +1366,7 @@ END
           |_| \_|\___|\__|     |_|   \__,_|_|_/_/   \_\__,_|\__\___/ ©
 
 
-   Copyright © 2000-2015  Brian M. Kelly  Brian.Kelly@FullAuto.com
+   Copyright © 2000-2016  Brian M. Kelly  Brian.Kelly@FullAuto.com
 
 END
    eval {
